@@ -38,8 +38,9 @@ DEFAULT_PAGE_SIZE = 10
 QUERY_PARAMETER_PAGE_SIZE = "page_size"
 QUERY_PARAMETER_PAGE_TOKEN = "page_token"
 
-# The Statement Phase List.
+
 class StatementPhase(StrEnum):
+    """This class defines the Flink SQL statement phases."""
     COMPLETED = "COMPLETED"
     DEGRADED = "DEGRADED"
     DELETED = "DELETED"
@@ -50,6 +51,11 @@ class StatementPhase(StrEnum):
     STOPPING = "STOPPING"
 
 
+class StatementType(StrEnum):
+    """This class defines the Flink SQL statement types."""
+    KAFKA_SINK = "INSERT_INTO"
+
+
 class FlinkClient():
     def __init__(self, flink_config: dict, private_network: bool = False):
         """This class initializes the Flink Client.
@@ -71,7 +77,7 @@ class FlinkClient():
         self.flink_sql_base_url = f"https://flink.{self.cloud_region}.{self.cloud_provider}.{'private.' if private_network else ''}confluent.cloud/sql/v1/organizations/{self.organization_id}/environments/{self.environment_id}/"
         self.flink_compute_pool_base_url = "https://api.confluent.cloud/fcpm/v2/compute-pools"
 
-    def statement_list(self, page_size: int = DEFAULT_PAGE_SIZE) -> Tuple[int, str, Dict]:
+    def get_statement_list(self, page_size: int = DEFAULT_PAGE_SIZE) -> Tuple[int, str, Dict]:
         """This function submits a RESTful API call to get the Flink SQL statement list.
 
         Arg(s):
@@ -161,73 +167,6 @@ class FlinkClient():
 
         return HttpStatus.ACCEPTED, ""
     
-    def stop_statement(self, statement_name: str, stop: bool = True) -> Tuple[int, str]:
-        """This function submits a RESTful API call to pause a Flink SQL statement.
-
-        Arg(s):
-            statement_name (str):  The Flink SQL statement name.
-            stop (bool):           (Optional) The stop flag. Default is True.
-
-        Returns:
-            int:    HTTP Status Code.
-            str:    HTTP Error, if applicable.
-        """
-        retry = 0
-        max_retries = 9
-        retry_delay_in_seconds = 5
-
-        while retry < max_retries:
-            # Send a GET request to get the statement.
-            response = requests.get(url=f"{self.flink_sql_base_url}statements/{statement_name}",
-                                    auth=HTTPBasicAuth(self.flink_api_key, self.flink_api_secret))
-
-            try:
-                # Raise HTTPError, if occurred.
-                response.raise_for_status()
-
-                # Turn the JSON response into a Statement model.
-                statement = Statement(**response.json())
-
-                # Get the statement resource version.
-                resource_version = statement.metadata.resource_version
-
-                # Set the stop flag.
-                statement.spec.stopped = stop
-
-                # Send a PUT request to update the status of the statement.
-                response = requests.put(url=f"{self.flink_sql_base_url}statements/{statement_name}",
-                                        data=statement.model_dump_json(),
-                                        auth=HTTPBasicAuth(self.flink_api_key, self.flink_api_secret))
-                
-                try:
-                    # Raise HTTPError, if occurred.
-                    response.raise_for_status()
-
-                    # Turn the JSON response into a Statement model.
-                    statement = Statement(**response.json())
-
-                    # Check if the resource version is the same.
-                    if statement.metadata.resource_version == resource_version:
-                        return response.status_code, response.text
-                    else:
-                        retry += 1
-                        if retry == max_retries:
-                            return response.status_code, f"Max retries exceeded.  Fail to update the statement because of a resource version mismatch.  Expected resource version #{resource_version}, but got resource version #{statement.metadata.resource_version}."
-                        else:
-                            time.sleep(retry_delay_in_seconds)
-                except requests.exceptions.RequestException as e:
-                    retry += 1
-                    if retry == max_retries:
-                        return response.status_code, f"Max retries exceeded.  Fail to update the statement because {e}, and the response is {response.text}"
-                    else:
-                        time.sleep(retry_delay_in_seconds)
-            except requests.exceptions.RequestException as e:
-                retry += 1
-                if retry == max_retries:
-                    return response.status_code, f"Max retries exceeded.  Fail to retrieve the statement because {e}, and the response is {response.text}"
-                else:
-                    time.sleep(retry_delay_in_seconds)
-
     def submit_statement(self, statement_name: str, sql_query: str, sql_query_properties: Dict) -> Tuple[int, str, Dict]:
         """This function submits a RESTful API call to submit a Flink SQL statement.
 
@@ -305,8 +244,7 @@ class FlinkClient():
                 return response.status_code, f"Fail to retrieve the computer pool because {e}", response.json() if response.content else {}
             
         return response.status_code, response.text, compute_pools
-        
-
+    
     def compute_pool(self) -> Tuple[int, str, Dict]:
         """This function submits a RESTful API call to get the Flink Compute Pool.
 
@@ -324,3 +262,151 @@ class FlinkClient():
                     return HttpStatus.OK, "", compute_pool
 
             return HttpStatus.NOT_FOUND, f"Fail to find the compute pool with ID {self.compute_pool_id}", response
+
+    def update_all_sink_statements(self, stop: bool = True, new_compute_pool_id: str = None, new_security_principal_id: str = None) -> Tuple[int, str]:
+        """This function submits a RESTful API call to update all Sink Flink SQL statements.
+        
+        Arg(s):
+            page_size (int):                 (Optional) The page size.
+            stop (bool):                     (Optional) The stop flag. Default is True.
+            new_compute_pool_id (str):       (Optional) The new compute pool ID.
+            new_security_principal_id (str): (Optional) The new security principal ID.
+            
+        Returns:
+            int:    HTTP Status Code.
+            str:    HTTP Error, if applicable.
+        """
+        # Get the statement list.
+        http_status_code, error_message, response = self.get_statement_list()
+
+        if http_status_code != HttpStatus.OK:
+            return http_status_code, error_message
+
+        # Update all background statements.
+        for response_item in response:
+            # Turn the JSON response into a Statement model.
+            statement = Statement(**response_item)
+
+            if statement.status.traits.sql_kind == StatementType.KAFKA_SINK:
+                http_status_code, error_message = self.update_statement(statement.name, stop=stop, new_compute_pool_id=new_compute_pool_id, new_security_principal_id=new_security_principal_id)
+
+                if http_status_code != HttpStatus.ACCEPTED:
+                    return http_status_code, error_message
+
+        return HttpStatus.ACCEPTED, ""
+
+    def update_statement(self, statement_name: str, stop: bool, new_compute_pool_id: str = None, new_security_principal_id: str = None) -> Tuple[int, str]:
+        """This function submits a RESTful API call to first stop the statement, and
+        then update the mutable attributes of a Flink SQL statement.
+
+        Arg(s):
+            statement_name (str):           The current Flink SQL statement name.
+            stop (bool):                    The stop flag.
+            new_compute_pool_id (str):      (Optional) The new compute pool ID.
+            new_security_principal_id (str):(Optional) The new security principal ID.
+
+        Returns:
+            int:    HTTP Status Code.
+            str:    HTTP Error, if applicable.
+        """
+        http_status_code, error_message = self.__update_statement(statement_name=statement_name, stop=True)
+        if http_status_code != HttpStatus.OK:
+            return http_status_code, error_message
+        else:
+            http_status_code, error_message = self.__update_statement(statement_name=statement_name, stop=stop, new_compute_pool_id=new_compute_pool_id, new_security_principal_id=new_security_principal_id)
+            return http_status_code, error_message
+
+    def stop_statement(self, statement_name: str, stop: bool = True) -> Tuple[int, str]:
+        """This function submits a RESTful API call to stop or start the Flink SQL statement.
+
+        For more information, why this function is a bit complex, please refer to the
+        Issue [#166](https://github.com/j3-signalroom/cc-clients-python_lib/issues/166).
+
+        Note: "Confluent Cloud for Apache Flink enforces a 30-day retention for statements in
+        terminal states." 
+
+        Arg(s):
+            statement_name (str):  The Flink SQL statement name.
+            stop (bool):           (Optional) The stop flag. Default is True.
+
+        Returns:
+            int:    HTTP Status Code.
+            str:    HTTP Error, if applicable.
+        """
+        return self.__update_statement(statement_name=statement_name, stop=stop)
+    
+    def __update_statement(self, statement_name: str, stop: bool, new_compute_pool_id: str = None, new_security_principal_id: str = None) -> Tuple[int, str]:
+        """This private function submits a RESTful API call to update the mutable attributes of a 
+        Flink SQL statement.
+
+        Arg(s):
+            statement_name (str):             The current Flink SQL statement name.
+            stop (bool):                      The stop flag.
+            new_compute_pool_id (str):        (Optional) The new compute pool ID.
+            new_security_principal_id (str):  (Optional) The new security principal ID.
+
+        Returns:
+            int:    HTTP Status Code.
+            str:    HTTP Error, if applicable.
+        """
+        retry = 0
+        max_retries = 9
+        retry_delay_in_seconds = 5
+
+        while retry < max_retries:
+            # Send a GET request to get the statement.
+            response = requests.get(url=f"{self.flink_sql_base_url}statements/{statement_name}",
+                                    auth=HTTPBasicAuth(self.flink_api_key, self.flink_api_secret))
+
+            try:
+                # Raise HTTPError, if occurred.
+                response.raise_for_status()
+
+                # Turn the JSON response into a Statement model.
+                statement = Statement(**response.json())
+
+                # Get the statement resource version.
+                resource_version = statement.metadata.resource_version
+
+                # Set the stop flag, compute pool ID, and security principal ID.
+                statement.spec.stopped = stop
+                if new_compute_pool_id is not None:
+                    statement.spec.compute_pool_id = new_compute_pool_id
+                if new_security_principal_id is not None:
+                    statement.spec.principal = new_security_principal_id
+
+                # Send a PUT request to update the status of the statement.
+                response = requests.put(url=f"{self.flink_sql_base_url}statements/{statement_name}",
+                                        data=statement.model_dump_json(),
+                                        auth=HTTPBasicAuth(self.flink_api_key, self.flink_api_secret))
+                
+                try:
+                    # Raise HTTPError, if occurred.
+                    response.raise_for_status()
+
+                    # Turn the JSON response into a Statement model.
+                    statement = Statement(**response.json())
+
+                    # Check if the resource version is the same.  If it is the same, the statement has successfully
+                    # been updated.  If it is not the same, this indicates that the statement has been updated since
+                    # the last GET request.  In this case, we need to retry the request.
+                    if statement.metadata.resource_version == resource_version:
+                        return response.status_code, response.text
+                    else:
+                        retry += 1
+                        if retry == max_retries:
+                            return response.status_code, f"Max retries exceeded.  Fail to update the statement because of a resource version mismatch.  Expected resource version #{resource_version}, but got resource version #{statement.metadata.resource_version}."
+                        else:
+                            time.sleep(retry_delay_in_seconds)
+                except requests.exceptions.RequestException as e:
+                    retry += 1
+                    if retry == max_retries:
+                        return response.status_code, f"Max retries exceeded.  Fail to update the statement because {e}, and the response is {response.text}"
+                    else:
+                        time.sleep(retry_delay_in_seconds)
+            except requests.exceptions.RequestException as e:
+                retry += 1
+                if retry == max_retries:
+                    return response.status_code, f"Max retries exceeded.  Fail to retrieve the statement because {e}, and the response is {response.text}"
+                else:
+                    time.sleep(retry_delay_in_seconds)
